// SARX Sierra SAR archive extractor

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <direct.h>

// dword into var
unsigned int getdword(unsigned char *_p)
{
	unsigned int ret;
	ret = (_p[3]<<24) + (_p[2]<<16) + (_p[1]<<8) + _p[0];
	return ret;
}

// string to number
int val(char* _str)
{
	int v=0;
	if (sscanf(_str,"0x%x",&v)!=1)
	{
		sscanf(_str,"%d",&v);
	}
	return v;
}

int main(int argc, char *argv[])
{
	unsigned char head[0x200];	// buffer for header
	char fname[FILENAME_MAX];
	char dname[FILENAME_MAX];
	unsigned int start;
	unsigned int end;
	unsigned int size;
	int fnum;
	int i,j;
	int maxfiles = 0x7f;

	if (argc<2)
	{
		fprintf(stderr, "Usage: %s [.SAR type file] [maxfiles]\n",argv[0]);
		return 1;
	}

	if (argc<2 || argc>4)
	{
		fprintf(stderr, "Usage: %s [.BND type file] [maxfiles]\n",argv[0]);
		return 1;
	}

	unsigned char *dp = head;

	// make directory
	strcpy(dname,argv[1]);
	*strrchr(dname,'.') = '\0';

	// file open
	FILE *fp1,*fp2;
	fp1 = fopen(argv[1],"rb");
	if (fp1==NULL)
	{ 
			fprintf(stderr, "Can't open file.\n");
			return 1;
	}

	// get file size
	fseek(fp1,0,SEEK_END); 
	int filesize = ftell(fp1);
	fseek(fp1,0,SEEK_SET); 

	// read header
	fread(head, 1, 0x200, fp1);

	// get maximum number of extracting files
	if (argc==3)
	{
		maxfiles = val(argv[2]);
	}
	else maxfiles = getdword(head)/4;

	// change dir
	_mkdir(dname);
	_chdir(dname);

	// file cut loop
	for (i=0; i<maxfiles; i++)
	{
		start = getdword(dp);
		if (i != maxfiles-1) {
			end = getdword(dp+4);
		} else end = filesize;
		size  = end-start;

		fseek(fp1,start,SEEK_SET);		// seek
		sprintf(fname,"%s.%03d",dname,i);	// make filename
		printf("cutting ... %-12s [%08x-%08x]\n",fname,start,end);
		fp2 = fopen(fname,"wb");
		for (j=0; j<size; j++)	// file cut
		{
			fputc(fgetc(fp1),fp2);
			if (ferror(fp2))
			{
				fprintf(stderr,"Write error.\n");
				fclose(fp1);
				fclose(fp2);
				return 2;
			}
		}
		if (fclose(fp2))
		{
			fprintf(stderr,"Write error.\n");
			fclose(fp1);
			return 2;
		}
		dp+=4;		// next file
	}
	fclose(fp1);
	_chdir("..");
	printf("Completed.\n");
	return 0;
}

